package org.implab.gradle.containers;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.gradle.api.logging.Logger;
import org.gradle.api.tasks.Internal;
import org.implab.gradle.containers.cli.Utils;

public interface ExecuteMixin {

    @Internal
    Logger getLogger();

    @Internal
    Optional<File> getWorkingDir();

    default void Execute() throws IOException, InterruptedException {
        final Logger log = getLogger();

        List<String> command = new ArrayList<>();

        preparingCommand(command);

        ProcessBuilder builder = new ProcessBuilder(command);
        getWorkingDir().ifPresent(workingDir -> builder.directory(workingDir));

        log.info("Starting: {}", builder.command());

        Process p = builder.start();

        processStarted(p);

        if (log.isErrorEnabled()) {
            Utils.redirectIO(p.getErrorStream(), log::error);
        }

        int code = p.waitFor();
        if (code != 0) {
            log.error("Process: {}", builder.command());
            log.error("Process exited with code {}", code);

            throw new IOException("Process exited with code: " + code);
        }
    }

    void preparingCommand(List<String> command);


    /**
     * Invoked after the process is started, can be used to handle the process
     * output.
     * 
     * Default implementation redirects output to the logger as INFO messages.
     * 
     * @param p The current process.
     */

    default void processStarted(Process p) {
        final Logger log = getLogger();
        Utils.redirectIO(p.getInputStream(), log::info);
    }
}
